﻿using Prova1.Model;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;
using System;

namespace Prova1Test
{


    /// <summary>
    ///This is a test class for ExamTest and is intended
    ///to contain all ExamTest Unit Tests
    ///</summary>
    [TestClass()]
    public class ExamTest
    {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext
        {
            get
            {
                return testContextInstance;
            }
            set
            {
                testContextInstance = value;
            }
        }

        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //[TestCleanup()]
        //public void MyTestCleanup()
        //{
        //}
        //
        #endregion

        /// <summary>
        ///A test for Exam Constructor
        ///</summary>
        [TestMethod()]
        [ExpectedException(typeof(ArgumentNullException))]
        public void ExamConstructorTest1()
        {
            Exam target = new Exam(null);
        }

        [TestMethod()]
        [ExpectedException(typeof(ArgumentException))]
        public void ExamConstructorTest2()
        {
            Exam target = new Exam(new List<Question>());
        }

        /// <summary>
        ///A test for CurrentQuestionIndex
        ///</summary>
        [TestMethod()]
        public void CurrentQuestionIndexTest()
        {
            IList<Question> questions = new List<Question> { new DummyQuestion(1), new DummyQuestion(2), new DummyQuestion(3) };
            Exam target = new Exam(questions);
            int expected = -1;
            Assert.AreEqual(expected, target.CurrentQuestionIndex);
            foreach (Question question in questions)
            {
                expected++;
                target.CurrentQuestionIndex++;
                Assert.AreEqual(expected, target.CurrentQuestionIndex);
            }
        }

        /// <summary>
        ///A test for CurrentQuestion
        ///</summary>
        [TestMethod()]
        public void CurrentQuestionTest()
        {
            IList<Question> questions = new List<Question> { new DummyQuestion(1), new DummyQuestion(2), new DummyQuestion(3) };
            Exam target = new Exam(questions);
            Assert.IsNull(target.CurrentQuestion);
            foreach (Question expected in questions)
            {
                target.CurrentQuestionIndex++;
                Assert.AreEqual(expected, target.CurrentQuestion);
            }
            target.CurrentQuestionIndex++;
            Assert.IsNull(target.CurrentQuestion);
        }

        /// <summary>
        ///A test for GetScore
        ///</summary>
        [TestMethod()]
        public void GetScoreTest1()
        {
            IList<Question> questions = new List<Question> { new DummyQuestion(1), new DummyQuestion(2), new DummyQuestion(3) };
            Exam target = new Exam(questions);
            double expected = 6;
            Assert.AreEqual(expected, target.GetScore());
        }

        [TestMethod()]
        public void GetScoreTest2()
        {
            IList<Question> questions = new List<Question> { new MultipleChoiceQuestion(new List<Answer>() { new Answer() }) };
            Exam target = new Exam(questions);
            Assert.AreEqual(0, target.GetScore());
        }
    }

    public class DummyQuestion : Question
    {
        public DummyQuestion(int id)
        {
            Id = "Q" + id.ToString();
            Score = id;
        }

        private int Score { get; set; }

        public override double GetScore()
        {
            return Score;
        }
    }
}
